
/*	Copyright (c) 2001, ATAPY Software. All rights reserved.

	This file is part of Pdf2Bmp. 
	
	Pdf2Bmp is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Pdf2Bmp is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "GS651Wrp.h" 

#include <StdIO.h> 

#include <Pdf2Bmp.h> 
#include <Tools.h> 

#include <shlobj.h> 

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

const int CGS651Wrapper::maxStringPartLength = 256; 
const int CGS651Wrapper::minRevisionVersion = 650; 

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

static int emptyCallback( int, char*, unsigned long ) 
{
	return 0; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

CGS651Wrapper::CGS651Wrapper() 
{
	reset(); 
}

bool CGS651Wrapper::Load( const char* fileName ) 
{
	return loadLibrary( fileName ); 
}

bool CGS651Wrapper::CheckRevision() const 
{
	long revision; 
	if( getRevision( 0, 0, &revision, 0 ) != 0 ) { 
		return FALSE; 
	}

	return revision >= minRevisionVersion; 
}

int CGS651Wrapper::InitializeInstance( GSDLL_CALLBACK callback, HWND hwnd, 
	int argc, char** argv ) const 
{
	return initializeInstance( callback, hwnd, argc, argv ); 
}

int CGS651Wrapper::DeinitializeInstance() const 
{
	return deinitializeInstance(); 
}

int CGS651Wrapper::RunScript( const char* script ) const 
{
	const int startCode = processStringStart(); 
	if( startCode != 0 ) { 
		return startCode; 
	}

	int length = strlen( script ); 
	while( maxStringPartLength < length ) { 
		const int continueCode = processStringContinue( 
			script, maxStringPartLength ); 
		if( continueCode != 0 ) { 
			return continueCode; 
		}

		length -= maxStringPartLength; 
		script += maxStringPartLength; 
	}

	const int continueCode = processStringContinue( 
		script, length ); 
	if( continueCode != 0 ) { 
		return continueCode; 
	}

	return processStringEnd(); 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

void CGS651Wrapper::reset() 
{
	hInstance = 0; 

	getRevisionFunc = 0; 
	initializeInstanceFunc = 0; 
	deinitializeInstanceFunc = 0; 

	processStringStartFunc = 0; 
	processStringContinueFunc = 0; 
	processStringEndFunc = 0; 
}

bool CGS651Wrapper::loadLibrary( const char* fileName ) 
{
	hInstance = ::LoadLibrary( fileName ); 
	if( hInstance == 0 ) { 
		return false; 
	}

	FARPROC getRevisionAddress = 
		::GetProcAddress( hInstance, "gsdll_revision" ); 
	if( getRevisionAddress == 0 ) { 
		return false; 
	}
	getRevisionFunc = 
		reinterpret_cast<GetRevisionFunc>( getRevisionAddress ); 

	FARPROC initializeInstanceAddress = 
		::GetProcAddress( hInstance, "gsdll_init" ); 
	if( initializeInstanceAddress == 0 ) { 
		return false; 
	}
	initializeInstanceFunc = 
		reinterpret_cast<InitializeInstanceFunc>( initializeInstanceAddress ); 

	FARPROC deinitializeInstanceAddress = 
		::GetProcAddress( hInstance, "gsdll_exit" ); 
	if( deinitializeInstanceAddress == 0 ) { 
		return false; 
	}
	deinitializeInstanceFunc = 
		reinterpret_cast<DeinitializeInstanceFunc>( deinitializeInstanceAddress ); 

	FARPROC processStringStartAddress = 
		::GetProcAddress( hInstance, "gsdll_execute_begin" ); 
	if( processStringStartAddress == 0 ) { 
		return false; 
	}
	processStringStartFunc = 
		reinterpret_cast<ProcessStringStartFunc>( processStringStartAddress ); 

	FARPROC processStringContinueAddress = 
		::GetProcAddress( hInstance, "gsdll_execute_cont" ); 
	if( processStringContinueAddress == 0 ) { 
		return false; 
	}
	processStringContinueFunc = 
		reinterpret_cast<ProcessStringContinueFunc>( processStringContinueAddress ); 

	FARPROC processStringEndAddress = 
		::GetProcAddress( hInstance, "gsdll_execute_end" ); 
	if( processStringEndAddress == 0 ) { 
		return false; 
	}
	processStringEndFunc = 
		reinterpret_cast<ProcessStringEndFunc>( processStringEndAddress ); 

	return true; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

int CGS651Wrapper::getRevision( char** product, char** copyright, 
	long* revision, long* date ) const 
{
	return getRevisionFunc( product, copyright, revision, date ); 
}

int CGS651Wrapper::initializeInstance( GSDLL_CALLBACK callback, HWND wnd, 
	int argc, char** argv ) const 
{
	return initializeInstanceFunc( callback, wnd, argc, argv ); 
}

int CGS651Wrapper::deinitializeInstance() const 
{
	return deinitializeInstanceFunc(); 
}

int CGS651Wrapper::processStringStart() const 
{
	return processStringStartFunc(); 
}

int CGS651Wrapper::processStringContinue( const char* string, int length ) const 
{
	errorStop( length <= maxStringPartLength ); 

	return processStringContinueFunc( string, length ); 
}

int CGS651Wrapper::processStringEnd() const 
{
	return processStringEndFunc(); 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

CGS651Parser::~CGS651Parser() 
{
}

CGS651Parser::CGS651Parser( const char* _fileName, 
		const char* _dllFolder, const char* _dllFileName ) 
	: fileName( _fileName ), dllFolder( _dllFolder ), dllFileName( _dllFileName ) 
{
}

int CGS651Parser::Load() 
{
	if( !wrapper.Load( dllFileName ) ) { 
		return err_CantLoadDLL; 
	}
	if( !wrapper.CheckRevision() ) { 
		return err_InvalidDLLRevision; 
	}

	return 0; 
}

int CGS651Parser::GetPageCount( const char* outputFileName ) const 
{
	try { 
		return getPageCount( outputFileName ); 
	} catch(...) { 
#ifdef _DEBUG
		throw; 	//	for debugging. 
#endif
		return err_UnknownError; 
	}
}

int CGS651Parser::OpenPage( int pageNumber, const char* outputFileName,
		const char* deviceName, int xResolution, int yResolution ) 
{
	try { 
		return openPage( pageNumber, outputFileName, deviceName, xResolution, yResolution ); 
	} catch(...) { 
#ifdef _DEBUG
		throw; 	//	for debugging. 
#endif
		return err_UnknownError; 
	}
}

int CGS651Parser::getPageCount( const char* outputFileName ) const 
{
	const int argc = 3; 
	char* argv[argc]; 
	argv[0] = const_cast<char*>( dllFileName ); 
	argv[1] = const_cast<char*>( createExtraPathString() ); 
	argv[2] = const_cast<char*>( createNoPauseString() ); 

	const int initCode = wrapper.InitializeInstance( 
		emptyCallback, 0, argc, argv ); 
	if( initCode != 0 ) { 
		return err_InvalidPDF; 
	}

	const char* script = createGetPageCountPostScriptString( 
		outputFileName ); 
	const int runCode = wrapper.RunScript( script ); 
	if( runCode != 0 ) { 
		wrapper.DeinitializeInstance(); 
		return err_InvalidPDF; 
	}

	wrapper.DeinitializeInstance(); 

	return 0; 
}

int CGS651Parser::openPage( int pageNumber, const char* outputFileName, 
		const char* deviceName, int xResolution, int yResolution ) const 
{
	const int argc = 8; 
	char* argv[argc]; 
	argv[0] = const_cast<char*>( dllFileName ); 
	argv[1] = const_cast<char*>( createExtraPathString() ); 
	argv[2] = const_cast<char*>( createNoPauseString() ); 
	argv[3] = const_cast<char*>( createOutputFormatString( deviceName ) ); 
	argv[4] = const_cast<char*>( createResolutionString( 'X', xResolution ) ); 
	argv[5] = const_cast<char*>( createResolutionString( 'Y', yResolution ) ); 
	argv[6] = const_cast<char*>( createOutputFileString( outputFileName ) ); 

	char* fontPath = const_cast<char*>( createExtraFontPathString() );
	argv[7] = fontPath;

	const int initCode = wrapper.InitializeInstance( 
		emptyCallback, 0, fontPath == 0 ? argc - 1 : argc, argv ); 
	if( initCode != 0 ) { 
		return err_InvalidPDF; 
	}

	const char* script = createOpenPagePostScriptString( 
		pageNumber ); 
	const int runCode = wrapper.RunScript( script ); 
	if( runCode != 0 ) { 
		wrapper.DeinitializeInstance(); 
		return err_InvalidPDF; 
	}

	wrapper.DeinitializeInstance(); 

	return 0; 
}

const char* CGS651Parser::createGetPageCountPostScriptString( 
	const char* outputFileName ) const 
{
	static const char* pageCountPostScriptString = 
		"%%!PS-Adobe 3.0 \r\n" 
		"pdfdict begin \r\n" 
		"(%s) (r) file pdfopen begin \r\n" 
		"/OutputFile (%s) (w) file def \r\n" 
		"pdfpagecount \r\n" 
		"dup string cvs OutputFile exch writestring \r\n" 
		"OutputFile closefile \r\n" 
		"currentdict pdfclose \r\n" 
		"end \r\n" 
		"end \r\n"; 

	int length = strlen( pageCountPostScriptString ); 

	length -= strlen( "%%" ); 
	length += strlen( "%" ); 

	length -= strlen( "%s" ); 
	length += strlen( fileName ) + getNumberOfSlash( fileName ); 

	length -= strlen( "%s" ); 
	length += strlen( outputFileName ) + getNumberOfSlash( outputFileName ); 

	char* ret = allocString( length ); 

	sprintf( ret, pageCountPostScriptString, 
		doubleSlashString( fileName ), doubleSlashString( outputFileName ) ); 
	int realLength = strlen( ret ); 
	errorStop( realLength == length ); 

	return ret; 
}

const char* CGS651Parser::createOpenPagePostScriptString( 
	int page ) const 
{
	const char* openPagePostScriptString = 
		"%%!PS-Adobe 3.0\r\n" 
		"/Page null def\r\n" 
		"/Page# 0 def\r\n" 
		"/PDFSave null def\r\n" 
		"/DSCPageCount 0 def\r\n" 
		"/DoPDFPage {\r\n" 
		"  (Page) print dup == flush\r\n" 
		"  dup /Page# exch store\r\n" 
		"  pdfgetpage pdfshowpage\r\n" 
		"} def\r\n" 
		"GS_PDF_ProcSet begin\r\n" 
		"pdfdict begin\r\n" 
		"(%s) (r) file pdfopen begin\r\n" 
		"%d DoPDFPage\r\n" 
		"currentdict pdfclose\r\n" 
		"end\r\n" 
		"end\r\n" 
		"end\r\n"; 

	int length = strlen( openPagePostScriptString ); 

	length -= strlen( "%%" ); 
	length += 1; 

	length -= strlen( "%s" ); 
	length += strlen( fileName ) + getNumberOfSlash( fileName ); 

	length -= 2 * strlen( "%d" ); 
	length += 2 * getStringLength( page + 1 ); 

	char* ret = allocString( length ); 

	sprintf( ret, openPagePostScriptString, 
		doubleSlashString( fileName ), page + 1, page + 1 ); 

	return ret; 
}

const char* CGS651Parser::createExtraPathString() const 
{
	char* ret = allocString( strlen( dllFolder ) + strlen( "-I" ) ); 

	sprintf( ret, "-I%s", dllFolder ); 

	return ret; 
}

////////////////////////////////////////////////////////////////////////////////////////////////

typedef HRESULT (__stdcall * TSHGetFolderPath)(
    HWND hwndOwner,
    int nFolder,
    HANDLE hToken,
    DWORD dwFlags,
    LPSTR pszPath
);

typedef BOOL (WINAPI *TSHGetPathFromIDList)(LPCITEMIDLIST pidl, LPSTR pszPath);

static const char* getSystemFontsFolder()
{
	static char windowsFontsFolder[MAX_PATH];

	TSHGetFolderPath shGetFolderPath = 0;

	HMODULE shellFolderModule = ::LoadLibrary( "shfolder.dll" );
	if( shellFolderModule != 0 ) {
		shGetFolderPath = (TSHGetFolderPath) ::GetProcAddress( shellFolderModule, "SHGetFolderPathA" );
		HRESULT hr = shGetFolderPath( 0, CSIDL_FONTS, 0, 0, windowsFontsFolder );
		if( SUCCEEDED(hr) ) {
			return windowsFontsFolder;
		}
	}

	IMalloc* pMalloc = 0;
	HRESULT hr = ::SHGetMalloc( &pMalloc );
	if( FAILED(hr) ) {
		return 0;
	}

    ITEMIDLIST* pidl = 0;
    hr = ::SHGetSpecialFolderLocation( NULL, CSIDL_FONTS, &pidl );
	if( FAILED(hr) ) {
		pMalloc->Release();
		return 0;
	}

	hr = ::SHGetPathFromIDList( pidl, windowsFontsFolder );
	pMalloc->Free( pidl );
	pMalloc->Release();

	if( SUCCEEDED(hr) ) {
		return windowsFontsFolder;
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////

const char* CGS651Parser::createExtraFontPathString() const
{
	const char* fontsFolder = getSystemFontsFolder();
	if( fontsFolder == 0 ) {
		return 0;
	}

	static const char* fontPathKey = "-sFONTPATH=";
	char* ret = allocString( strlen( fontsFolder ) + strlen( fontPathKey ) ); 
	sprintf( ret, "%s%s", fontPathKey, fontsFolder );
	return ret;
}

const char* CGS651Parser::createNoPauseString() const 
{
	return "-dNOPAUSE -dNOGC";
}

const char* CGS651Parser::createOutputFormatString( const char* deviceName ) const 
{
	char* ret = allocString( strlen( "-sDEVICE=" ) + strlen( deviceName ) );
	sprintf( ret, "-sDEVICE=%s", deviceName );
	return ret; 
}

const char* CGS651Parser::createResolutionString( 
	char ch, int resolution ) const 
{
	char* ret = allocString( getStringLength( resolution ) + strlen( "-dDEVICE RESOLUTION=" ) ); 

	sprintf( ret, "-dDEVICE%cRESOLUTION=%d", ch, resolution ); 

	return ret; 
}

const char* CGS651Parser::createOutputFileString( 
	const char* outputFileName ) const 
{
	char* ret = allocString( strlen( outputFileName ) + strlen( "-sOutputFile=" ) ); 

	sprintf( ret, "-sOutputFile=%s" , outputFileName ); 

	return ret; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
