
/*	Copyright (c) 2001, ATAPY Software. All rights reserved.

	This file is part of Pdf2Bmp. 
	
	Pdf2Bmp is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Pdf2Bmp is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Windows.h> 
#include <String.h> 
#include <StdIO.h> 
#include <Math.h> 

#include <GS651Wrp.h> 
#include <GS700Wrp.h> 
#include <Tools.h> 
#include "Pdf2Bmp.h" 

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

static const char* getGSDLLFolder() 
{
	const int maxPathSize = 4096; 
	char fileName[maxPathSize]; 

	HINSTANCE hInstance = ::GetModuleHandle( 0 ); 
	const int length = ::GetModuleFileName( hInstance, fileName, maxPathSize ); 
	errorStop( length > 0 ); 

	char drive[_MAX_DRIVE]; 
	char dir[_MAX_DIR]; 
	_splitpath( fileName, drive, dir, 0, 0 ); 

	char* ret = allocString( strlen( drive ) + strlen( dir ) ); 
	sprintf( ret, "%s%s", drive, dir ); 

	return ret; 
}

static const char* getGSDLLName( const char* folder ) 
{
	char* ret = allocString( strlen( folder ) + strlen( "gsdll32.dll" ) ); 
	sprintf( ret, "%s%s", folder, "gsdll32.dll" ); 

	return ret; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

static int parseCommandLine( char* cmdLine, 
	char** argv, int maxArgc ) 
{
	if( maxArgc <= 0 ) { 
		return 0; 
	}

	for( ; isspace( *cmdLine ); cmdLine++ ) { 
		; 
	}
	if( *cmdLine == 0 ) { 
		return 0; 
	}

	bool isQuoted = false; 
	for( char* p = cmdLine; *p != 0; p++ ) { 
		if( !isQuoted && isspace( *p ) ) { 
			*argv = cmdLine; 
			*p = 0; 

			return parseCommandLine( p + 1, argv + 1, maxArgc - 1 ) + 1; 
		} else if( *p == '\"' ) { 
			isQuoted = !isQuoted; 
		}
	}
	*argv = cmdLine; 

	return 1; 
}

static const char* findArg( const char* prefix, 
	int argc, char** argv ) 
{
	for( int i = 0; i < argc; i++ ) { 
		if( strstr( argv[i], prefix ) == argv[i] ) { 
			return argv[i] + strlen( prefix ); 
		}
	}

	return 0; 
}

static const char* getFileName( const char* string ) 
{
	const int length = strlen( string ); 
	if( string[0] == '\"' && string[length - 1] == '\"' ) { 
		char* dup = _strdup( string ); 
		dup[length - 1] = 0; 

		return dup + 1; 
	} else { 
		return string; 
	}
}

static bool getInt( const char* string, int& value ) 
{
	errno = 0; 
	char* endString; 
	value = int( strtol( string, &endString, 10 ) ); 
	if( errno == ERANGE || endString == string ) { 
		return false; 
	} else { 
		return *endString == 0; 
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

static int realMain( int argc, char** argv ) 
{
	const char* gsdllFolder = getGSDLLFolder(); 
	const char* gsdllName = getGSDLLName( gsdllFolder ); 

	const char* command = findArg( "/c:", argc, argv ); 
	if( command == 0 ) { 
		return err_InvalidParams; 
	} else { 
		const char* srcFile = findArg( "/s:", argc, argv ); 
		if( srcFile == 0 ) { 
			return err_InvalidParams; 
		}
		const char* srcFileName = getFileName( srcFile ); 

		const char* dstFile = findArg( "/d:", argc, argv ); 
		if( dstFile == 0 ) { 
			return err_InvalidParams; 
		}
		const char* dstFileName = getFileName( dstFile ); 

		if( strcmp( command, "count" ) == 0 ) { 
			//CGS651Parser parser( srcFileName, gsdllFolder, gsdllName ); 
			CGS700Parser parser( srcFileName, gsdllFolder, gsdllName ); 
			const int loadCode = parser.Load(); 
			if( loadCode != 0 ) { 
				return loadCode; 
			}

			return parser.GetPageCount( dstFileName ); 
		} else if( strcmp( command, "print" ) == 0 ) { 
			const char* pageNumber = findArg( "/p:", argc, argv ); 
			if( pageNumber == 0 ) { 
				return err_InvalidParams; 
			}
			int page; 
			if( !getInt( pageNumber, page ) ) { 
				return err_InvalidParams; 
			}

			const char* formatName = findArg( "/f:", argc, argv ); 
			if( formatName == 0 ) {
				return err_InvalidParams;
			}

			const char* xResolution = findArg( "/rx:", argc, argv ); 
			if( xResolution == 0 ) { 
				return err_InvalidParams; 
			}
			int xRes; 
			if( !getInt( xResolution, xRes ) ) { 
				return err_InvalidParams; 
			}

			const char* yResolution = findArg( "/ry:", argc, argv ); 
			if( yResolution == 0 ) { 
				return err_InvalidParams; 
			}
			int yRes; 
			if( !getInt( yResolution, yRes ) ) { 
				return err_InvalidParams; 
			}

			//CGS651Parser parser( srcFileName, gsdllFolder, gsdllName ); 
			CGS700Parser parser( srcFileName, gsdllFolder, gsdllName ); 
			const int loadCode = parser.Load(); 
			if( loadCode != 0 ) { 
				return loadCode; 
			}

			return parser.OpenPage( page, dstFileName, formatName, xRes, yRes ); 
		} else { 
			return err_InvalidParams; 
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

int APIENTRY WinMain( HINSTANCE /*hInstance*/, 
	HINSTANCE /*hPrevInstance*/, LPSTR lpCmdLine, int /*nCmdShow*/ ) 
{
	char* cmdLine = allocString( strlen( lpCmdLine ) ); 
	strcpy( cmdLine, lpCmdLine ); 

	const int maxArgc = 7; 
	char* argv[maxArgc]; 
	const int argc = parseCommandLine( cmdLine, argv, maxArgc ); 

	return realMain( argc, argv ); 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
